-- Initialise SimpleCalc
function SimpleCalc_OnLoad(self)

  -- Register our slash commands
  SLASH_SIMPLECALC1="/simplecalc";
  SLASH_SIMPLECALC2="/calc";
  SlashCmdList["SIMPLECALC"]=SimpleCalc_ParseParameters;

  -- Let the user know that we're here
  DEFAULT_CHAT_FRAME:AddMessage("[+] SimpleCalc initiated! Type: /calc for help.", 1, 1, 1);

end


-- Parse any user-passed parameters
function SimpleCalc_ParseParameters(paramStr)

  local params={};

  paramStr=string.lower(paramStr);

  local i=0;
  local a=nil;
  local b=nil;
  local result=0;
  local paramCount=0;
  for param in string.gmatch(paramStr, "[^%s]+") do

    i=i+1;
    paramCount=paramCount+1;

    -- Make sure that we've been passed a number or a variable
    if (i==1 or i==3) then
      -- Check whether we have a number
      if (string.match(param, '[^%d\.-]')) then
        -- If we have something other than a number, see whether it's a recognised variable
        if (param=='honour' or param=='honor') then
          param=GetHonorCurrency();
        elseif (param=='arenapoints') then
          param=GetArenaCurrency();
        elseif (param=='health') then
          param=UnitHealthMax('player')
        elseif (param=='mana') then
          param=UnitManaMax('player')
        elseif (param=='copper') then
          param=GetMoney();
        elseif (param=='silver') then
          param=GetMoney() / 100;
        elseif (param=='gold') then
          param=GetMoney() / 10000;
        else
          SimpleCalc_Error(paramStr .. ': \'' .. param .. '\' isn\'t a recognised variable, and doesn\'t look like a number!');
          return false;
        end
      end
    end

    if (i==1) then
      a=tonumber(param);
    elseif (i==2) then
      symbol=param;
    elseif (i==3) then
      b=tonumber(param);

      -- Perform the operation
      -- Can't find any way around doing things this way...
      if (symbol=='+') then
        result=a + b;
      elseif (symbol=='-') then
        result=a - b;
      elseif (symbol=='*') then
        result=a * b;
      elseif (symbol=='/') then
        result=a / b;
      elseif (symbol=='^') then
        result=a ^ b;
      else
        SimpleCalc_Error('Unrecognised symbol: ' .. symbol);
        if (symbol=='x') then
          SimpleCalc_Error('Perhaps you meant '*' (multiply)?');
        end
        return false;
      end

      -- Reset for another loop
      i=1;
      symbol=nil;
      a=result;
      b=nil;

    end

  end

  if (symbol and not b) then
    SimpleCalc_Error('Unbalanced parameter count. Trailing symbol: ' .. symbol .. '?');
  end

  if (paramCount >= 3) then
    SimpleCalc_Message(paramStr .. ' = ' .. number_format(result));
  else
    SimpleCalc_Usage();
  end

end


-- Inform the user of our their options
function SimpleCalc_Usage()
  SimpleCalc_Message("SimpleCalc - Simple mathematical calculator - http://addons.guildworks.co.uk/");
  SimpleCalc_Message("Usage: /calc <value> <symbol> <value> ...");
  SimpleCalc_Message("Example: 14500 + 10500 - honour");
  SimpleCalc_Message("value - A numeric value or variable (supported variables: honour, arenapoints, health, mana, copper, silver, gold)");
  SimpleCalc_Message("symbol - A mathematical symbol (supported symbols: +, -, /, *, ^)");
end


-- Output errors
function SimpleCalc_Error(message)
  DEFAULT_CHAT_FRAME:AddMessage("[SimpleCalc] " .. message, 0.8, 0.2, 0.2);
end


-- Output messages
function SimpleCalc_Message(message)
  DEFAULT_CHAT_FRAME:AddMessage("[SimpleCalc] " .. message, 0.4, 0.4, 1);
end


-- Number formatting function, taken from http://lua-users.org/wiki/FormattingNumbers
function number_format(amount)
  local formatted=amount;
  while true do  
    formatted, k=string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1,%2');
    if (k==0) then
      break;
    end
  end
  return formatted;
end

